@@ -412,9 +412,6 @@ eglGetDisplay(EGLNativeDisplayType nativeDisplay)
    _EGLDisplay *disp;
    void *native_display_ptr;
 
-#if !DETECT_OS_ANDROID
-   util_cpu_trace_init();
-#endif
    _EGL_FUNC_START(NULL, EGL_OBJECT_THREAD_KHR, NULL);
 
    STATIC_ASSERT(sizeof(void *) >= sizeof(nativeDisplay));
@@ -479,9 +476,6 @@ eglGetPlatformDisplayEXT(EGLenum platform, void *native_display,
    EGLAttrib *attrib_list;
    EGLDisplay disp;
 
-#if !DETECT_OS_ANDROID
-   util_cpu_trace_init();
-#endif
    _EGL_FUNC_START(NULL, EGL_OBJECT_THREAD_KHR, NULL);
 
    if (_eglConvertIntsToAttribs(int_attribs, &attrib_list) != EGL_SUCCESS)
@@ -496,9 +490,6 @@ PUBLIC EGLDisplay EGLAPIENTRY
 eglGetPlatformDisplay(EGLenum platform, void *native_display,
                       const EGLAttrib *attrib_list)
 {
-#if !DETECT_OS_ANDROID
-   util_cpu_trace_init();
-#endif
    _EGL_FUNC_START(NULL, EGL_OBJECT_THREAD_KHR, NULL);
    return _eglGetPlatformDisplayCommon(platform, native_display, attrib_list);
 }
@@ -603,11 +594,9 @@ _eglCreateExtensionsString(_EGLDisplay *disp)
    _EGL_CHECK_EXTENSION(MESA_query_driver);
    _EGL_CHECK_EXTENSION(MESA_x11_native_visual_id);
 
-   _EGL_CHECK_EXTENSION(NOK_swap_region);
    _EGL_CHECK_EXTENSION(NOK_texture_from_pixmap);
 
    _EGL_CHECK_EXTENSION(NV_context_priority_realtime);
-   _EGL_CHECK_EXTENSION(NV_post_sub_buffer);
 
    _EGL_CHECK_EXTENSION(WL_bind_wayland_display);
    _EGL_CHECK_EXTENSION(WL_create_wayland_buffer_from_image);
@@ -675,7 +664,6 @@ eglInitialize(EGLDisplay dpy, EGLint *major, EGLint *minor)
 {
    _EGLDisplay *disp = _eglLockDisplay(dpy);
 
-   util_cpu_trace_init();
    _EGL_FUNC_START(disp, EGL_OBJECT_DISPLAY_KHR, NULL);
 
    _eglDeviceRefreshList();
@@ -709,9 +697,7 @@ eglInitialize(EGLDisplay dpy, EGLint *major, EGLint *minor)
             bool success = false;
             if (!disp->Options.Zink && !getenv("GALLIUM_DRIVER")) {
                disp->Options.Zink = EGL_TRUE;
-               disp->Options.FallbackZink = EGL_TRUE;
                success = _eglDriver.Initialize(disp);
-               disp->Options.FallbackZink = EGL_FALSE;
             }
             if (!success) {
                disp->Options.Zink = EGL_FALSE;
@@ -2250,33 +2236,6 @@ eglDupNativeFenceFDANDROID(EGLDisplay dpy, EGLSync sync)
    RETURN_EGL_SUCCESS(disp, ret);
 }
 
-static EGLBoolean EGLAPIENTRY
-eglSwapBuffersRegionNOK(EGLDisplay dpy, EGLSurface surface, EGLint numRects,
-                        const EGLint *rects)
-{
-   _EGLContext *ctx = _eglGetCurrentContext();
-   _EGLDisplay *disp = _eglLockDisplay(dpy);
-   _EGLSurface *surf = _eglLookupSurface(surface, disp);
-   EGLBoolean ret = EGL_FALSE;
-
-   _EGL_FUNC_START(disp, EGL_OBJECT_SURFACE_KHR, surf);
-
-   _EGL_CHECK_SURFACE(disp, surf, EGL_FALSE);
-
-   if (!disp->Extensions.NOK_swap_region)
-      RETURN_EGL_EVAL(disp, EGL_FALSE);
-
-   /* surface must be bound to current context in EGL 1.4 */
-   if (_eglGetContextHandle(ctx) == EGL_NO_CONTEXT || surf != ctx->DrawSurface)
-      RETURN_EGL_ERROR(disp, EGL_BAD_SURFACE, EGL_FALSE);
-
-   egl_relax (disp, &surf->Resource) {
-      ret = disp->Driver->SwapBuffersRegionNOK(disp, surf, numRects, rects);
-   }
-
-   RETURN_EGL_EVAL(disp, ret);
-}
-
 static EGLImage EGLAPIENTRY
 eglCreateDRMImageMESA(EGLDisplay dpy, const EGLint *attr_list)
 {
@@ -2408,28 +2367,6 @@ eglCreateWaylandBufferFromImageWL(EGLDisplay dpy, EGLImage image)
    RETURN_EGL_EVAL(disp, ret);
 }
 
-static EGLBoolean EGLAPIENTRY
-eglPostSubBufferNV(EGLDisplay dpy, EGLSurface surface, EGLint x, EGLint y,
-                   EGLint width, EGLint height)
-{
-   _EGLDisplay *disp = _eglLockDisplay(dpy);
-   _EGLSurface *surf = _eglLookupSurface(surface, disp);
-   EGLBoolean ret = EGL_FALSE;
-
-   _EGL_FUNC_START(disp, EGL_OBJECT_SURFACE_KHR, surf);
-
-   _EGL_CHECK_SURFACE(disp, surf, EGL_FALSE);
-
-   if (!disp->Extensions.NV_post_sub_buffer)
-      RETURN_EGL_EVAL(disp, EGL_FALSE);
-
-   egl_relax (disp, &surf->Resource) {
-      ret = disp->Driver->PostSubBufferNV(disp, surf, x, y, width, height);
-   }
-
-   RETURN_EGL_EVAL(disp, ret);
-}
-
 static EGLBoolean EGLAPIENTRY
 eglGetSyncValuesCHROMIUM(EGLDisplay dpy, EGLSurface surface, EGLuint64KHR *ust,
                          EGLuint64KHR *msc, EGLuint64KHR *sbc)
