@@ -54,6 +54,8 @@ struct ir3_compiler_options {
 
    /* "dual_color_blend_by_location" workaround is enabled: */
    bool dual_color_blend_by_location;
+
+   uint64_t uche_trap_base;
 };
 
 struct ir3_compiler {
@@ -431,6 +433,10 @@ ir3_shader_debug_hash_key()
         IR3_DBG_SHADER_INTERNAL | IR3_DBG_SCHEDMSGS | IR3_DBG_RAMSGS));
 }
 
+/* Returns a pointer to internal static tmp buffer. */
+const char *
+ir3_shader_debug_as_string(void);
+
 ENDC;
 
 #endif /* IR3_COMPILER_H_ */
