@@ -87,6 +87,11 @@ void
 etna_compile_rs_state(struct etna_context *ctx, struct compiled_rs_state *cs,
                       const struct rs_state *rs);
 
+void
+etna_align_box_for_rs(const struct etna_screen *screen,
+                      const struct etna_resource *rsc,
+                      struct pipe_box *box);
+
 /* Context initialization for RS clear_blit functions. */
 void
 etna_clear_blit_rs_init(struct pipe_context *pctx);
