@@ -43,6 +43,13 @@ struct nv50_miptree_level {
    uint32_t tile_mode;
 };
 
+struct nv50_memobj {
+   struct winsys_handle *handle;
+   struct pipe_memory_object b;
+   struct nouveau_bo *bo;
+   unsigned stride;
+};
+
 #define NV50_MAX_TEXTURE_LEVELS 16
 
 struct nv50_miptree {
@@ -66,6 +73,7 @@ nv50_miptree(struct pipe_resource *pt)
 #define NV50_TEXVIEW_SCALED_COORDS     (1 << 0)
 #define NV50_TEXVIEW_FILTER_MSAA8      (1 << 1)
 #define NV50_TEXVIEW_ACCESS_RESOLVE    (1 << 2)
+#define NV50_TEXVIEW_IMAGE_GM107       (1 << 3)
 
 
 /* Internal functions:
@@ -85,10 +93,12 @@ nv50_miptree_from_handle(struct pipe_screen *pscreen,
                          const struct pipe_resource *template,
                          struct winsys_handle *whandle);
 
-boolean
+bool
 nv50_miptree_get_handle(struct pipe_screen *pscreen,
+                        struct pipe_context *context,
                         struct pipe_resource *pt,
-                        struct winsys_handle *whandle);
+                        struct winsys_handle *whandle,
+                        unsigned usage);
 
 struct nv50_surface {
    struct pipe_surface base;
@@ -151,4 +161,29 @@ nv50_surface_from_buffer(struct pipe_context *pipe,
 void
 nv50_surface_destroy(struct pipe_context *, struct pipe_surface *);
 
+void
+nv50_invalidate_resource(struct pipe_context *, struct pipe_resource *);
+
+void
+nv50_clear_texture(struct pipe_context *pipe,
+                   struct pipe_resource *res,
+                   unsigned level,
+                   const struct pipe_box *box,
+                   const void *data);
+
+struct pipe_memory_object *
+nv50_memobj_create_from_handle(struct pipe_screen *screen,
+                               struct winsys_handle *handle,
+                               bool dedicated);
+
+struct pipe_resource *
+nv50_resource_from_memobj(struct pipe_screen *screen,
+                          const struct pipe_resource *t,
+                          struct pipe_memory_object *pmemobj,
+                          uint64_t offset);
+
+void
+nv50_memobj_destroy(struct pipe_screen *screen,
+                    struct pipe_memory_object *pmemobj);
+
 #endif /* __NV50_RESOURCE_H__ */
