@@ -53,8 +53,7 @@ struct virgl_rasterizer_state {
 };
 
 struct virgl_shader_binding_state {
-   struct pipe_sampler_view *views[16];
-   uint32_t view_enabled_mask;
+   struct pipe_sampler_view *views[PIPE_MAX_SHADER_SAMPLER_VIEWS];
 
    struct pipe_constant_buffer ubos[PIPE_MAX_CONSTANT_BUFFERS];
    uint32_t ubo_enabled_mask;
@@ -85,10 +84,11 @@ struct virgl_context {
    struct virgl_staging_mgr staging;
    bool encoded_transfers;
    bool supports_staging;
+   uint8_t patch_vertices;
 
    struct pipe_vertex_buffer vertex_buffer[PIPE_MAX_ATTRIBS];
    unsigned num_vertex_buffers;
-   boolean vertex_array_dirty;
+   bool vertex_array_dirty;
 
    struct virgl_rasterizer_state rs_state;
    struct virgl_so_target so_targets[PIPE_MAX_SO_BUFFERS];
@@ -103,6 +103,13 @@ struct virgl_context {
    uint64_t queued_staging_res_size;
 };
 
+struct virgl_vertex_elements_state {
+   uint32_t handle;
+   uint8_t binding_map[PIPE_MAX_ATTRIBS];
+   uint8_t num_bindings;
+   uint16_t strides[PIPE_MAX_ATTRIBS];
+};
+
 static inline struct virgl_sampler_view *
 virgl_sampler_view(struct pipe_sampler_view *view)
 {
@@ -127,7 +134,8 @@ void virgl_init_blit_functions(struct virgl_context *vctx);
 void virgl_init_query_functions(struct virgl_context *vctx);
 void virgl_init_so_functions(struct virgl_context *vctx);
 
-struct tgsi_token *virgl_tgsi_transform(struct virgl_screen *vscreen, const struct tgsi_token *tokens_in);
+struct tgsi_token *virgl_tgsi_transform(struct virgl_screen *vscreen, const struct tgsi_token *tokens_in,
+                                        bool is_separable);
 
 bool
 virgl_can_rebind_resource(struct virgl_context *vctx,
@@ -137,4 +145,6 @@ void
 virgl_rebind_resource(struct virgl_context *vctx,
                       struct pipe_resource *res);
 
+void virgl_flush_eq(struct virgl_context *ctx, void *closure, struct pipe_fence_handle **fence);
+
 #endif
