@@ -31,6 +31,7 @@
 #include "util/u_blitter.h"
 #include "util/format/u_format.h"
 #include "util/format_srgb.h"
+#include "util/helpers.h"
 #include "util/u_framebuffer.h"
 #include "util/u_inlines.h"
 #include "util/u_rect.h"
@@ -187,7 +188,7 @@ zink_clear(struct pipe_context *pctx,
                color.f[3] = 1.0;
                pctx->clear_render_target(pctx, ctx->fb_state.cbufs[i], &color,
                                          0, 0,
-                                         ctx->fb_state.cbufs[i]->width, ctx->fb_state.cbufs[i]->height,
+                                         ctx->fb_state.width, ctx->fb_state.height,
                                          ctx->render_condition_active);
             }
             pctx->clear_render_target(pctx, ctx->fb_state.cbufs[i], pcolor,
@@ -416,8 +417,13 @@ static void
 set_clear_fb(struct pipe_context *pctx, struct pipe_surface *psurf, struct pipe_surface *zsurf)
 {
    struct pipe_framebuffer_state fb_state = {0};
-   fb_state.width = psurf ? psurf->width : zsurf->width;
-   fb_state.height = psurf ? psurf->height : zsurf->height;
+   uint16_t width, height;
+   if (psurf)
+      pipe_surface_size(psurf, &width, &height);
+   else
+      pipe_surface_size(zsurf, &width, &height);
+   fb_state.width = width;
+   fb_state.height = height;
    fb_state.nr_cbufs = !!psurf;
    fb_state.cbufs[0] = psurf;
    fb_state.zsbuf = zsurf;
@@ -744,17 +750,24 @@ zink_fb_clear_reset(struct zink_context *ctx, unsigned i)
       ctx->rp_loadop_changed = true;
 }
 
+static bool
+fb_depth_intersects(const struct pipe_surface *psurf, int z, int depth)
+{
+   return (z >= psurf->u.tex.first_layer && z + depth - 1 <= psurf->u.tex.last_layer) ||
+          (psurf->u.tex.first_layer >= z && psurf->u.tex.last_layer <= z + depth - 1);
+}
+
 void
-zink_fb_clears_apply(struct zink_context *ctx, struct pipe_resource *pres)
+zink_fb_clears_apply(struct zink_context *ctx, struct pipe_resource *pres, int z, int depth)
 {
    if (zink_resource(pres)->aspect == VK_IMAGE_ASPECT_COLOR_BIT) {
       for (int i = 0; i < ctx->fb_state.nr_cbufs; i++) {
-         if (ctx->fb_state.cbufs[i] && ctx->fb_state.cbufs[i]->texture == pres) {
+         if (ctx->fb_state.cbufs[i] && ctx->fb_state.cbufs[i]->texture == pres && fb_depth_intersects(ctx->fb_state.cbufs[i], z, depth)) {
             fb_clears_apply_internal(ctx, pres, i);
          }
       }
    } else {
-      if (ctx->fb_state.zsbuf && ctx->fb_state.zsbuf->texture == pres) {
+      if (ctx->fb_state.zsbuf && ctx->fb_state.zsbuf->texture == pres && fb_depth_intersects(ctx->fb_state.zsbuf, z, depth)) {
          fb_clears_apply_internal(ctx, pres, PIPE_MAX_COLOR_BUFS);
       }
    }
@@ -805,11 +818,11 @@ zink_clear_apply_conditionals(struct zink_context *ctx)
 }
 
 static void
-fb_clears_apply_or_discard_internal(struct zink_context *ctx, struct pipe_resource *pres, struct u_rect region, bool discard_only, bool invert, int i)
+fb_clears_apply_or_discard_internal(struct zink_context *ctx, struct pipe_resource *pres, struct u_rect region, bool discard_only, bool invert, bool depth_fills, int i)
 {
    struct zink_framebuffer_clear *fb_clear = &ctx->fb_clears[i];
    if (zink_fb_clear_enabled(ctx, i)) {
-      if (zink_blit_region_fills(region, pres->width0, pres->height0)) {
+      if (zink_blit_region_fills(region, pres->width0, pres->height0) && depth_fills) {
          if (invert)
             fb_clears_apply_internal(ctx, pres, i);
          else
@@ -821,7 +834,7 @@ fb_clears_apply_or_discard_internal(struct zink_context *ctx, struct pipe_resour
          struct zink_framebuffer_clear_data *clear = zink_fb_clear_element(fb_clear, j);
          struct u_rect scissor = {clear->scissor.minx, clear->scissor.maxx,
                                   clear->scissor.miny, clear->scissor.maxy};
-         if (!clear->has_scissor || zink_blit_region_covers(region, scissor)) {
+         if (!clear->has_scissor || zink_blit_region_covers(region, scissor) || !depth_fills) {
             /* this is a clear that isn't fully covered by our pending write */
             if (!discard_only)
                fb_clears_apply_internal(ctx, pres, i);
@@ -834,34 +847,42 @@ fb_clears_apply_or_discard_internal(struct zink_context *ctx, struct pipe_resour
    }
 }
 
+static bool
+fb_depth_fills(const struct pipe_surface *psurf, int z, int depth)
+{
+   return z == psurf->u.tex.first_layer && z + depth - 1 >= psurf->u.tex.last_layer;
+}
+
 void
-zink_fb_clears_apply_or_discard(struct zink_context *ctx, struct pipe_resource *pres, struct u_rect region, bool discard_only)
+zink_fb_clears_apply_or_discard(struct zink_context *ctx, struct pipe_resource *pres, struct u_rect region, int z, int depth, bool discard_only)
 {
    if (zink_resource(pres)->aspect == VK_IMAGE_ASPECT_COLOR_BIT) {
       for (int i = 0; i < ctx->fb_state.nr_cbufs; i++) {
-         if (ctx->fb_state.cbufs[i] && ctx->fb_state.cbufs[i]->texture == pres) {
-            fb_clears_apply_or_discard_internal(ctx, pres, region, discard_only, false, i);
+         if (ctx->fb_state.cbufs[i] && ctx->fb_state.cbufs[i]->texture == pres &&
+             fb_depth_intersects(ctx->fb_state.cbufs[i], z, depth)) {
+            fb_clears_apply_or_discard_internal(ctx, pres, region, discard_only, false, fb_depth_fills(ctx->fb_state.cbufs[i], z, depth), i);
          }
       }
    }  else {
-      if (zink_fb_clear_enabled(ctx, PIPE_MAX_COLOR_BUFS) && ctx->fb_state.zsbuf && ctx->fb_state.zsbuf->texture == pres) {
-         fb_clears_apply_or_discard_internal(ctx, pres, region, discard_only, false, PIPE_MAX_COLOR_BUFS);
+      if (zink_fb_clear_enabled(ctx, PIPE_MAX_COLOR_BUFS) && ctx->fb_state.zsbuf && ctx->fb_state.zsbuf->texture == pres &&
+          fb_depth_intersects(ctx->fb_state.zsbuf, z, depth)) {
+         fb_clears_apply_or_discard_internal(ctx, pres, region, discard_only, false, fb_depth_fills(ctx->fb_state.zsbuf, z, depth), PIPE_MAX_COLOR_BUFS);
       }
    }
 }
 
 void
-zink_fb_clears_apply_region(struct zink_context *ctx, struct pipe_resource *pres, struct u_rect region)
+zink_fb_clears_apply_region(struct zink_context *ctx, struct pipe_resource *pres, struct u_rect region, int z, int depth)
 {
    if (zink_resource(pres)->aspect == VK_IMAGE_ASPECT_COLOR_BIT) {
       for (int i = 0; i < ctx->fb_state.nr_cbufs; i++) {
-         if (ctx->fb_state.cbufs[i] && ctx->fb_state.cbufs[i]->texture == pres) {
-            fb_clears_apply_or_discard_internal(ctx, pres, region, false, true, i);
+         if (ctx->fb_state.cbufs[i] && ctx->fb_state.cbufs[i]->texture == pres && fb_depth_intersects(ctx->fb_state.cbufs[i], z, depth)) {
+            fb_clears_apply_or_discard_internal(ctx, pres, region, false, true, fb_depth_fills(ctx->fb_state.cbufs[i], z, depth), i);
          }
       }
    }  else {
-      if (ctx->fb_state.zsbuf && ctx->fb_state.zsbuf->texture == pres) {
-         fb_clears_apply_or_discard_internal(ctx, pres, region, false, true, PIPE_MAX_COLOR_BUFS);
+      if (ctx->fb_state.zsbuf && ctx->fb_state.zsbuf->texture == pres && fb_depth_intersects(ctx->fb_state.zsbuf, z, depth)) {
+         fb_clears_apply_or_discard_internal(ctx, pres, region, false, true, fb_depth_fills(ctx->fb_state.zsbuf, z, depth), PIPE_MAX_COLOR_BUFS);
       }
    }
 }
