@@ -47,9 +47,11 @@ VkImageLayout
 zink_tc_renderpass_info_parse(struct zink_context *ctx, const struct tc_renderpass_info *info, unsigned idx, VkPipelineStageFlags *pipeline, VkAccessFlags *access);
 bool
 zink_init_render_pass(struct zink_context *ctx);
-void
+bool
 zink_render_update_swapchain(struct zink_context *ctx);
 void
+zink_render_fixup_swapchain(struct zink_context *ctx);
+void
 zink_init_zs_attachment(struct zink_context *ctx, struct zink_rt_attrib *rt);
 void
 zink_init_color_attachment(struct zink_context *ctx, unsigned i, struct zink_rt_attrib *rt);
@@ -57,4 +59,6 @@ void
 zink_tc_init_zs_attachment(struct zink_context *ctx, const struct tc_renderpass_info *info, struct zink_rt_attrib *rt);
 void
 zink_tc_init_color_attachment(struct zink_context *ctx, const struct tc_renderpass_info *info, unsigned i, struct zink_rt_attrib *rt);
+void
+zink_render_msaa_expand(struct zink_context *ctx, uint32_t msaa_expand_mask);
 #endif
