@@ -27,7 +27,7 @@ const struct nvk_va_format *
 nvk_get_va_format(const struct nvk_physical_device *pdev, VkFormat format);
 
 static inline enum pipe_format
-nvk_format_to_pipe_format(enum VkFormat vkformat)
+nvk_format_to_pipe_format(VkFormat vkformat)
 {
    switch (vkformat) {
    case VK_FORMAT_R10X6_UNORM_PACK16:
