@@ -379,6 +379,34 @@ vk_image_to_memory_copy_layout(const struct vk_image *image,
    return vk_image_buffer_copy_layout(image, &bic);
 }
 
+bool
+vk_image_can_be_aliased_to_yuv_plane(const struct vk_image *image)
+{
+   if (!(image->create_flags & VK_IMAGE_CREATE_ALIAS_BIT))
+      return false;
+
+   VkFormat format = image->format;
+
+   /* Only the 8-bit, 16-bit, and 32-bit classes listed in
+    * https://registry.khronos.org/vulkan/specs/latest/html/vkspec.html#formats-compatibility-classes
+    * are compatible with yuv planes. We must exclude other classes with the
+    * same block size as these.
+    */
+   if (vk_format_is_depth_or_stencil(format) ||
+       vk_format_is_alpha(format) ||
+       vk_format_get_blockwidth(format) != 1 ||
+       vk_format_get_blockheight(format) != 1)
+      return false;
+
+   unsigned block_size = vk_format_get_blocksize(format);
+
+   /* The planes of all the multiplane formats have a block size of 1, 2, or 4.
+    * See:
+    * https://registry.khronos.org/vulkan/specs/latest/html/vkspec.html#formats-compatible-planes
+    */
+   return block_size == 1 || block_size == 2 || block_size == 4;
+}
+
 static VkComponentSwizzle
 remap_swizzle(VkComponentSwizzle swizzle, VkComponentSwizzle component)
 {
